library(DiceKriging)


#RUN OPTIONS>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
var <- "temperature" #temperature or precipitation
which <- "djf" #mean, max, min, djf, mam, jja, son

ncomp <- 10 #number of components in emulator - up to 10 enabled
sdscale1 <- 0.0 #=0 use emulator mean prediction =1 standard GP uncertainty: 1st components scaling
sdscale <- 0.0 #=0 use emulator mean prediction =1 standard GP uncertainty: higher order component scaling

covar="powexp" #matern3_2,matern5_2,gauss,exp,powexp 

cval=0 # =0 for time series (or ensemble) emulation or =1 to cross validate E1 (CO2 and orbit) emulator
cvalice=0 # =0 to zero for time series (or ensemble) emulation or =1 to cross validate E2 (ice) emulator

#forcing parameters (either for time series or for a fixed boundary condition ensemble)
forcing <- as.data.frame(read.table("5000_1000_forcing.dat",header=TRUE))
#forcing <- as.data.frame(read.table("MH_forcing.dat",header=TRUE))    #for MH ensemble
#forcing <- as.data.frame(read.table("LGM_forcing.dat",header=TRUE))   #for LGM ensemble
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

centre_ice <- 0
if(cvalice==0) centre_ice <- 1

size <- 50

#ensemble parameters
params <- as.data.frame(read.table("ensemble.dat",header=TRUE))
params <- params[-c(39),] #remove PCA outlier
paramsi <- params         #copy for ice sheet emulator
params <- params[,-c(1)]  #remove ice sheet forcing in CO2 ORBIT emulator


forcingi <- forcing #copy for ice sheet emulator
forcing <- forcing[,-c(1)] #remove ice sheet forcing in CO2 ORBIT emulator

#make CO2 & orbit forced emulator
djfdata <- scan(paste("data/DJF_",var,".dat",sep=""))
mamdata <- scan(paste("data/MAM_",var,".dat",sep=""))
jjadata <- scan(paste("data/JJA_",var,".dat",sep=""))
sondata <- scan(paste("data/SON_",var,".dat",sep=""))
if(which=="mean") data <- (djfdata+mamdata+jjadata+sondata)*0.25
if(which=="max")  data <- pmax(djfdata,mamdata,jjadata,sondata)
if(which=="min")  data <- pmin(djfdata,mamdata,jjadata,sondata)
if(which=="djf")  data <- djfdata
if(which=="mam")  data <- mamdata
if(which=="jja")  data <- jjadata
if(which=="son")  data <- sondata

dim(data) <- c(size,2048)
data <- t(data)
mean <- rowMeans(data)
data <- data-mean #centre data
data <- data[,-c(39)] #remove PCA outlier
svddata <- svd(data,nu=10,nv=10) #perform singular vector decomposition

if(cval==1) {
# CROSS VALIDATE
size1 <- dim(svddata$v)[1]
em <- rep(0,10*size1)
sim <- rep(0,10*size1)
dim(em) <- c(10,size1)
dim(sim) <- c(10,size1)

for (i in 1:10)
{
 for (j in 1:size1)
  { 
  traini <- params[-j,]
  traino <- svddata$v[-j,i]
  testi <- params[j,]
  testo <- svddata$v[j,i]

  GP <- km(formula ~.,
         design=traini,
         response=traino,
         nugget.estim=FALSE,
         covtype=covar) #build GP emulator
  tmp <- predict(GP,
        newdata=testi,
        type="UK") #predict left out simulation
  em[i,j]  <- tmp$mean
  sim[i,j] <- testo
 }
}
score <- 0
for (i in 1:10) {
	print(c(i,cor(em[i,],sim[i,]),svddata$d[i]**2/sum(svddata$d**2)))
        score <- score +
                (cor(em[i,],sim[i,])**2) *
                (svddata$d[i]**2/sum(svddata$d**2))
}
print(score) #P score
print(nonsense)
}
#build GP emulators for time series reconstruction
  GP1 <- km(formula ~.,design=params,response=svddata$v[,1],covtype=covar)
  GP2 <- km(formula ~.,design=params,response=svddata$v[,2],covtype=covar)
  GP3 <- km(formula ~.,design=params,response=svddata$v[,3],covtype=covar)
  GP4 <- km(formula ~.,design=params,response=svddata$v[,4],covtype=covar)
  GP5 <- km(formula ~.,design=params,response=svddata$v[,5],covtype=covar)
  GP6 <- km(formula ~.,design=params,response=svddata$v[,6],covtype=covar)
  GP7 <- km(formula ~.,design=params,response=svddata$v[,7],covtype=covar)
  GP8 <- km(formula ~.,design=params,response=svddata$v[,8],covtype=covar)
  GP9 <- km(formula ~.,design=params,response=svddata$v[,9],covtype=covar)
  GP10 <- km(formula ~.,design=params,response=svddata$v[,10],covtype=covar)

#make ice forced emulator

djfdata <- scan(paste("icedata/DJF_",var,".dat",sep=""))
mamdata <- scan(paste("icedata/MAM_",var,".dat",sep=""))
jjadata <- scan(paste("icedata/JJA_",var,".dat",sep=""))
sondata <- scan(paste("icedata/SON_",var,".dat",sep=""))
if(which=="mean") icedata <- (djfdata+mamdata+jjadata+sondata)*0.25
if(which=="max")  icedata <- pmax(djfdata,mamdata,jjadata,sondata)
if(which=="min")  icedata <- pmin(djfdata,mamdata,jjadata,sondata)
if(which=="djf")  icedata <- djfdata
if(which=="mam")  icedata <- mamdata
if(which=="jja")  icedata <- jjadata
if(which=="son")  icedata <- sondata

dim(icedata) <- c(size,2048)
icedata <- t(icedata)
icedata <- icedata[,-c(39)]
icedata <- icedata - data - mean
if(centre_ice==1) icedata <- icedata - rowMeans(icedata)
svdicedata <- svd(icedata,nu=10,nv=10)


if(cvalice==1) {
# CROSS VALIDATE
size1 <- dim(svdicedata$v)[1]
em <- rep(0,10*size1)
sim <- rep(0,10*size1)
dim(em) <- c(10,size1)
dim(sim) <- c(10,size1)

for (i in 1:10)
{
 for (j in 1:size1)
  { 
  traini <- paramsi[-j,]
  traino <- svdicedata$v[-j,i]
  testi <- paramsi[j,]
  testo <- svdicedata$v[j,i]

  GP <- km(formula ~.,
         design=traini,
         response=traino,
         nugget.estim=FALSE,
         covtype=covar)
  tmp <- predict(GP,
        newdata=testi,
        type="UK")
  em[i,j]  <- tmp$mean
  sim[i,j] <- testo
 }
}
score <- 0
for (i in 1:10) {
	print(c(i,cor(em[i,],sim[i,]),svdicedata$d[i]**2/sum(svdicedata$d**2)))
        score <- score +
                (cor(em[i,],sim[i,])**2) *
                (svdicedata$d[i]**2/sum(svdicedata$d**2))
}
print(score) #P score
print(nonsense)
}

#double ensmble size with zeroed outputs for no ice sheet forcing
size1 <- dim(icedata)[2]
icedata <- cbind(icedata,icedata)
paramsi <- rbind(paramsi,paramsi)
icedata[,(size1+1):(size1*2)] <- 0
paramsi[(size1+1):(size1*2),1] <- runif(size1)*1.25-0.25
svdicedata <- svd(icedata,nu=10,nv=10)

#build GP emulators for time series reconstruction
  GPi1 <- km(formula ~.,design=paramsi,response=svdicedata$v[,1],covtype=covar)
  GPi2 <- km(formula ~.,design=paramsi,response=svdicedata$v[,2],covtype=covar)
  GPi3 <- km(formula ~.,design=paramsi,response=svdicedata$v[,3],covtype=covar)
  GPi4 <- km(formula ~.,design=paramsi,response=svdicedata$v[,4],covtype=covar)
  GPi5 <- km(formula ~.,design=paramsi,response=svdicedata$v[,5],covtype=covar)
  GPi6 <- km(formula ~.,design=paramsi,response=svdicedata$v[,6],covtype=covar)
  GPi7 <- km(formula ~.,design=paramsi,response=svdicedata$v[,7],covtype=covar)
  GPi8 <- km(formula ~.,design=paramsi,response=svdicedata$v[,8],covtype=covar)
  GPi9 <- km(formula ~.,design=paramsi,response=svdicedata$v[,9],covtype=covar)
  GPi10 <- km(formula ~.,design=paramsi,response=svdicedata$v[,10],covtype=covar)

weights <- scan("area.dat")
weights <- rep(weights,64)
weights <- as.matrix(weights)
dim(weights) <- c(1,2048)

emulmat <- c()

nslice <- dim(forcing)[1]-0

PC1 <- predict(GP1,newdata=forcing,type="UK")
PC2 <- predict(GP2,newdata=forcing,type="UK")
PC3 <- predict(GP3,newdata=forcing,type="UK")
PC4 <- predict(GP4,newdata=forcing,type="UK")
PC5 <- predict(GP5,newdata=forcing,type="UK")
PC6 <- predict(GP6,newdata=forcing,type="UK")
PC7 <- predict(GP7,newdata=forcing,type="UK")
PC8 <- predict(GP8,newdata=forcing,type="UK")
PC9 <- predict(GP9,newdata=forcing,type="UK")
PC10 <- predict(GP10,newdata=forcing,type="UK")
PCi1 <- predict(GPi1,newdata=forcingi,type="UK")
PCi2 <- predict(GPi2,newdata=forcingi,type="UK")
PCi3 <- predict(GPi3,newdata=forcingi,type="UK")
PCi4 <- predict(GPi4,newdata=forcingi,type="UK")
PCi5 <- predict(GPi5,newdata=forcingi,type="UK")
PCi6 <- predict(GPi6,newdata=forcingi,type="UK")
PCi7 <- predict(GPi7,newdata=forcingi,type="UK")
PCi8 <- predict(GPi8,newdata=forcingi,type="UK")
PCi9 <- predict(GPi9,newdata=forcingi,type="UK")
PCi10 <- predict(GPi10,newdata=forcingi,type="UK")

PC <- cbind(
            rnorm(nslice,mean=PC1$mean,sd=PC1$sd*sdscale1),
            rnorm(nslice,mean=PC2$mean,sd=PC2$sd*sdscale)*(2<=ncomp),
            rnorm(nslice,mean=PC3$mean,sd=PC3$sd*sdscale)*(3<=ncomp),
            rnorm(nslice,mean=PC4$mean,sd=PC4$sd*sdscale)*(4<=ncomp),
            rnorm(nslice,mean=PC5$mean,sd=PC5$sd*sdscale)*(5<=ncomp),
            rnorm(nslice,mean=PC6$mean,sd=PC6$sd*sdscale)*(6<=ncomp),
            rnorm(nslice,mean=PC7$mean,sd=PC7$sd*sdscale)*(7<=ncomp),
            rnorm(nslice,mean=PC8$mean,sd=PC8$sd*sdscale)*(8<=ncomp),
            rnorm(nslice,mean=PC9$mean,sd=PC9$sd*sdscale)*(9<=ncomp),
            rnorm(nslice,mean=PC10$mean,sd=PC10$sd*sdscale)*(10<=ncomp)
           )
PCi <- cbind(
            rnorm(nslice,mean=PCi1$mean,sd=PCi1$sd*sdscale1),
            rnorm(nslice,mean=PCi2$mean,sd=PCi2$sd*sdscale)*(2<=ncomp),
            rnorm(nslice,mean=PCi3$mean,sd=PCi3$sd*sdscale)*(3<=ncomp),
            rnorm(nslice,mean=PCi4$mean,sd=PCi4$sd*sdscale)*(4<=ncomp),
            rnorm(nslice,mean=PCi5$mean,sd=PCi5$sd*sdscale)*(5<=ncomp),
            rnorm(nslice,mean=PCi6$mean,sd=PCi6$sd*sdscale)*(6<=ncomp),
            rnorm(nslice,mean=PCi7$mean,sd=PCi7$sd*sdscale)*(7<=ncomp),
            rnorm(nslice,mean=PCi8$mean,sd=PCi8$sd*sdscale)*(8<=ncomp),
            rnorm(nslice,mean=PCi9$mean,sd=PCi9$sd*sdscale)*(9<=ncomp),
            rnorm(nslice,mean=PCi10$mean,sd=PCi10$sd*sdscale*(10<=ncomp))
           )

D <- diag(svddata$d[1:10])
Di <- diag(svdicedata$d[1:10])

emul <- svddata$u[,1:10] %*% D %*% t(PC[,1:10]) + 
        svdicedata$u[,1:10] %*% Di %*% t(PCi[,1:10]) +
        mean

for (i in 1:nslice) {
print(c(i,(weights %*% emul[,i])/sum(weights)))
}

emul <- signif(emul,digits=4)
write(t(emul),"emul.dat",nslice)


RowSD <- function(x) {
  sqrt(rowSums((x - rowMeans(x))^2)/(dim(x)[2] - 1))
}

emul_mean <- rowMeans(emul[,1:nslice-1]-emul[,nslice])
emul_SD <- RowSD(emul[,1:nslice-1]-emul[,nslice])

write(emul_mean,"mean.dat",1) #ensemble mean
write(emul_SD,"SD.dat",1)    #ensemble mean


